// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AssetManager.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

void AssetManager::init(const std::shared_ptr<Game> &game) {
  assertNotNull(game, "Game is NULL?");
  this->game = game;
}

void AssetManager::update() {
  auto itPending = pendingAssetLoaders.begin();
  while(itPending != pendingAssetLoaders.end()) {
    auto loader = *itPending;
    loader->updateSync();
    loader->updateAsync();
    loader->updateSync();
    if(loader->loaded) {
      finishedAssetLoaders.push_back(loader);
      itPending = pendingAssetLoaders.erase(itPending);
    } else {
      itPending++;
    }
  }
}

void AssetManager::remove(const std::shared_ptr<AssetLoader> loader) {
  for(
    auto it = pendingAssetLoaders.begin();
    it != pendingAssetLoaders.end();
    it++
  ) {
    if(*it != loader) continue;
    pendingAssetLoaders.erase(it);
    return;
  }

  for(
    auto it = finishedAssetLoaders.begin();
    it != finishedAssetLoaders.end();
    it++
  ) {
    if(it->lock() != loader) continue;
    finishedAssetLoaders.erase(it);
    return;
  }
}

bool_t AssetManager::isEverythingLoaded() {
  return pendingAssetLoaders.size() == 0;
}

std::shared_ptr<Game> AssetManager::getGame() {
  auto g = game.lock();
  assertNotNull(g, "Game is NULL?");
  return g;
}

AssetManager::~AssetManager() {
  
}